package com.gx.obe.common.file.download.infer;

import java.text.DecimalFormat;
import java.util.Calendar;

import com.gx.obe.message.Messages;

/**
 * @Description: 下载监听
 * @author chenxw
 */
public abstract class FileDownloadListener {

	private long startTime = 0;
	private long useTime = 0;
	private double downLoadTotalSize = 0;
	private double oldDownLoadTotalSize = 0;
	private double hisDownloadSize = 0;
	private double totalSize = 0;
	private DOWN_STATUS downloadStatus = DOWN_STATUS.WAITE;

	private static DecimalFormat fmt = new DecimalFormat("##,###,###,###,##0.00");

	public abstract void startDownload();
	
	public abstract void downloadSuccess();

	public abstract void downloadFailed();

	public enum DOWN_STATUS {
		/**
		 * 等待，开始，暂停，成功，失败，取消
		 */
		WAITE, START, PAUSE, SUCCESS, FAIL, CANCEL
	}

	public synchronized void setDownloadStatus(DOWN_STATUS downloadStatus) {
		this.downloadStatus = downloadStatus;
		if (downloadStatus == DOWN_STATUS.START) {
			startTime = Calendar.getInstance().getTimeInMillis();
			startDownload();
		} else if (downloadStatus == DOWN_STATUS.PAUSE) {
			useTime += Calendar.getInstance().getTimeInMillis() - startTime;
		} else if (downloadStatus == DOWN_STATUS.SUCCESS) {
			downloadSuccess();
		} else if (downloadStatus == DOWN_STATUS.FAIL) {
			downloadFailed();
		}
	}

	public DOWN_STATUS getDownloadStatus() {
		return downloadStatus;
	}

	/**
	 * @Description:重置
	 * @author chenxw
	 */
	public void reset() {
		downloadStatus = DOWN_STATUS.WAITE;
		startTime = 0;
		useTime = 0;
		downLoadTotalSize = 0;
		oldDownLoadTotalSize = 0;
		hisDownloadSize = 0;
		totalSize = 0;
	}

	/**
	 * @Description: 获得总文件大小
	 * @author chenxw
	 * @return
	 */
	public double getTotalSize() {
		return totalSize;
	}

	/**
	 * @Description: 写入总文件大小
	 * @author chenxw
	 * @param totalSize
	 */
	public void setTotalSize(double totalSize) {
		this.totalSize = totalSize;
	}

	/**
	 * @Description: 写入下载长度
	 * @author chenxw
	 * @param size
	 */
	public synchronized void writeSize(long size) {
		downLoadTotalSize += size;
	}

	/**
	 * @Description: 写入历史下载大小
	 * @author chenxw
	 * @param size
	 */
	public synchronized void hisDownloadSize(long size) {
		hisDownloadSize += size;
	}

	/**
	 * @Description: 获取下载进度
	 * @author chenxw
	 * @return
	 */
	public double getProgress() {
		if (totalSize == 0) {
			return 0;
		}
		return ((downLoadTotalSize + hisDownloadSize) / totalSize) * 100;
	}

	/**
	 * @Description: 获取下载进度
	 * @author chenxw
	 * @return
	 */
	public String getProgressString() {
		if (totalSize == 0) {
			return "0%";
		}
		return (int) (((downLoadTotalSize + hisDownloadSize) / totalSize) * 100) + "%";
	}

	/**
	 * @Description: 获得已下载文件大小
	 * @author chenxw
	 * @return
	 */
	public String getDownloadedShare() {
		if (totalSize == 0) {
			return Messages.Unknown + Messages.Length;
		}
		return computeSize((downLoadTotalSize + hisDownloadSize)).concat("/").concat(computeSize(totalSize));
	}

	/**
	 * @Description: 获得用时
	 * @author chenxw
	 * @return
	 */
	public String getRuntime() {
		long runtime = 0;
		if (downloadStatus == DOWN_STATUS.WAITE) {
			return computeTime(runtime);
		}
		if (downloadStatus == DOWN_STATUS.PAUSE) {
			runtime = useTime;
		} else {
			runtime = Calendar.getInstance().getTimeInMillis() - startTime + useTime;
		}
		return computeTime(runtime);
	}

	/**
	 * @Description: 获得下载速度
	 * @author chenxw
	 * @param minsecond
	 * @return
	 */
	public String getDownloadSpeed(long minsecond) {
		double speed = (downLoadTotalSize - oldDownLoadTotalSize) / minsecond * 1000;
		oldDownLoadTotalSize = downLoadTotalSize;
		return computeSize(speed).concat("/S");
	}

	/**
	 * @Description: 获得下载完成后文件大小
	 * @author chenxw
	 * @return
	 */
	public String getFileSize() {
		return computeSize(downLoadTotalSize + hisDownloadSize);
	}

	/**
	 * @Description: 计算文件大小
	 * @author chenxw
	 * @param size
	 * @return
	 */
	public static String computeSize(double size) {
		String unit = "KB";
		if (size < 0)
			size = 0;
		if (size > (1024 * 1024 * 1024)) {
			unit = "GB";
			size = (double) size / (1024 * 1024 * 1024);
		} else if (size > (1024 * 1024)) {
			unit = "MB";
			size = (double) size / (1024 * 1024);
		} else if (size > (1024)) {
			unit = "KB";
			size = (double) size / (1024);
		} else {
			unit = "字节";
		}
		return fmt.format(size).concat(unit);
	}

	/**
	 * @Description: 计算用时
	 * @author chenxw
	 * @param msUsed
	 * @return
	 */
	public static String computeTime(long msUsed) {
		if (msUsed < 1000) {
			return "小于1秒";
		}
		msUsed /= 1000;
		if (msUsed < 60) {
			return msUsed + "秒";
		}
		if (msUsed < 3600) {
			long nMinute = msUsed / 60;
			long nSecond = msUsed % 60;
			return nMinute + "分" + nSecond + "秒";
		}
		if (msUsed < 86400) {
			long nHour = msUsed / 3600;
			long nMinute = (msUsed - nHour * 3600) / 60;
			long nSecond = (msUsed - nHour * 3600) % 60;
			return nHour + "时" + nMinute + "分" + nSecond + "秒";
		}
		return "大于24小时";
	}

	/**
	 * @Description: 计算下载速度
	 * @author chenxw
	 * @param length
	 * @param time
	 * @return
	 */
	public static String computeSpeed(double length, long time) {
		double speed = length * 1024 * 1024 / (double) (time == 0 ? 1 : time * 1000);
		String unit = "KB";
		if (speed < 0)
			speed = 0;
		if (speed > (1024 * 1024 * 1024)) {
			unit = "GB";
			speed = (double) speed / (1024 * 1024 * 1024);
		} else if (speed > (1024 * 1024)) {
			unit = "MB";
			speed = (double) speed / (1024 * 1024);
		} else if (speed > (1024)) {
			unit = "KB";
			speed = (double) speed / (1024);
		} else {
			unit = "字节";
		}
		return fmt.format(speed).concat(unit).concat("/S");
	}

}
