package com.gx.obe.common.file.entity;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.RandomAccessFile;

import org.apache.log4j.Logger;

import com.gx.obe.util.utils.FileUtils;

/**
 * @Description: 下载分块对象
 * @author guoyr
 */
public class DownBlock {

	private String downURL;

	private String saveFile;

	private int bufferSize = 2 * 1024 * 1024;
	
	private int threadCount = 10;// 线程池的容量，如果正在运行的线程达到了上限，则会阻塞，直到有了空闲的线程来运行。

	private int blockCount = 10;// 文件要切分的块数，每个切块将启动一个线程

	private long contentLength;// 文件的大小

	private long[] sectionsOffset;
	
	private long downLoadLength;// 已下载长度
	
	private String downLoadSpeed;// 下载速度
	
	private long progress;// 下载完成百分比
	
	private long downLoadTime; // 下载用时
	
	private String md5;// 文件的md5值

	//在任务描述文件里，我把前4K用来做一些描述，然后之后的用于记录下载的过程
	public static final int HEAD_SIZE = 4096;
	
	private Logger LOG = Logger.getLogger(DownBlock.class);
	public DownBlock(String downURL, String savePath) {
		this.downURL = downURL;
		this.saveFile = savePath;
	}

	// 读下载描述文件内容
	public synchronized void read(RandomAccessFile file) throws IOException {
		byte[] temp = new byte[HEAD_SIZE];
		file.seek(0);
		int readed = file.read(temp);
		if (readed != temp.length) {
//			throw new RuntimeException();
			LOG.error("描述文件出错");
		}
		ByteArrayInputStream bais = new ByteArrayInputStream(temp);
		DataInputStream dis = new DataInputStream(bais);
		downURL = dis.readUTF();
		saveFile = dis.readUTF();
		blockCount = dis.readInt();
		contentLength = dis.readLong();
		sectionsOffset = new long[blockCount];

		for (int i = 0; i < blockCount; i++) {
			sectionsOffset[i] = file.readLong();
		}
	}

	/** 
	 * @Description: 创建并记录描述文件 
	 * @author guoyr
	 * @param   
	 * @return  
	 * @Exception   
	 */
	public synchronized void create(RandomAccessFile file) throws IOException {
		if (blockCount != sectionsOffset.length) {
			throw new RuntimeException();
		}
		long len = HEAD_SIZE + 8 * blockCount;
		file.setLength(len);
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		DataOutputStream dos = new DataOutputStream(baos);
		dos.writeUTF(downURL);
		dos.writeUTF(saveFile);
		dos.writeInt(blockCount);
		dos.writeLong(contentLength);
		byte[] src = baos.toByteArray();
		byte[] temp = new byte[HEAD_SIZE];
		System.arraycopy(src, 0, temp, 0, src.length);
		file.seek(0);
		file.write(temp);
		writeOffset(file);

	}

	/** 
	 * @Description: 记录描述文件的内容 和更新下载的过程
	 * @author guoyr
	 * @param   
	 * @return  
	 * @Exception   
	 */
	public synchronized void writeOffset(RandomAccessFile file)
			throws IOException {
		if (blockCount != sectionsOffset.length) {
			throw new RuntimeException();
		}
		
		//前4K用于记录描述，从4K之后记录下载的过程
		file.seek(HEAD_SIZE);
		for (int i = 0; i < sectionsOffset.length; i++) {
			file.writeLong(sectionsOffset[i]);
		}
	}

	public String getDownURL() {
		return downURL;
	}

	public void setDownURL(String downURL) {
		this.downURL = downURL;
	}

	public String getSaveFile() {
		return saveFile;
	}

	public void setSaveFile(String saveFile) {
		this.saveFile = saveFile;
	}

	public int getBufferSize() {
		return bufferSize;
	}

	public void setBufferSize(int bufferSize) {
		this.bufferSize = bufferSize;
	}

	public int getThreadCount() {
		return threadCount;
	}

	public void setThreadCount(int threadCount) {
		this.threadCount = threadCount;
	}

	public int getBlockCount() {
		return blockCount;
	}

	public void setBlockCount(int blockCount) {
		this.blockCount = blockCount;
	}

	public long getContentLength() {
		return contentLength;
	}

	public void setContentLength(long contentLength) {
		// 如果大小在2M内，就不分块进行下载
		if((contentLength/(double)(1024*1024)) <= 10){
			this.blockCount = 1;
		}else if((contentLength/(double)(1024*1024)) > 10 && (contentLength/(double)(1024*1024)) <= 30){
			this.blockCount = 2;
		}else if((contentLength/(double)(1024*1024)) > 30 && (contentLength/(double)(1024*1024)) <= 80){
			this.blockCount = 4;
		}else if((contentLength/(double)(1024*1024)) > 80 && (contentLength/(double)(1024*1024)) <= 120){
			this.blockCount = 6;
		}else if((contentLength/(double)(1024*1024)) > 120 && (contentLength/(double)(1024*1024)) <= 160){
			this.blockCount = 8;
		}else if((contentLength/(double)(1024*1024)) > 160){
			blockCount = 10;//Integer.valueOf(PropertiesUtils.getLabel("config.properties", "maxDownThread", "10"));
			if(blockCount >= 10){
				blockCount = 10;
			}
//			this.blockCount = Integer.parseInt((contentLength/(1024*1024*10))+"");
		}
		this.threadCount = blockCount;
		LOG.debug("将大小为"+FileUtils.getFileSize(contentLength)+"的文件，切分成"+blockCount+"个分块进行下载");
		this.contentLength = contentLength;
	}

	public long[] getSectionsOffset() {
		return sectionsOffset;
	}

	public void setSectionsOffset(long[] sectionsOffset) {
		this.sectionsOffset = sectionsOffset;
	}

	public long getDownLoadLength() {
		return downLoadLength;
	}

	public void setDownLoadLength(long downLoadLength) {
		this.downLoadLength = downLoadLength;
	}

	public String getDownLoadSpeed() {
		return downLoadSpeed;
	}

	public long getProgress() {
		if (this.contentLength > 0) {
			progress = Math.round(this.downLoadLength * 100 / (double) this.contentLength);
		} else {
			progress = 0L;
		}
		return progress;
	}

	public void setProgress(long progress) {
		this.progress = progress;
	}

	public void setDownLoadSpeed(String downLoadSpeed) {
		this.downLoadSpeed = downLoadSpeed;
	}

	public long getDownLoadTime() {
		return downLoadTime;
	}

	public void setDownLoadTime(long downLoadTime) {
		this.downLoadTime = downLoadTime;
	}

	public String getMd5() {
		return md5;
	}

	public void setMd5(String md5) {
		this.md5 = md5;
	}
}
